import { MaybeFactory, MaybePromise } from '../utils/types';
import { ServiceEventEmitter, ServiceEventSubscriber } from './event';
import { ServiceInstance } from './service-instance';
import { InferServiceID, Service, ServiceInterface } from './service-interface';
import { InferModuleID, ServiceModule, ServiceModuleRegistry } from './service-module-registry';
import { ServiceRegistry } from './service-registry';
export declare function createServiceGroup(): ServiceGroup;
export declare class ServiceGroup {
    _services: {
        [serviceId: string]: MaybeFactory<Service>;
    };
    _modules: {
        [serviceId: string]: MaybeFactory<ServiceModule>;
    };
    _eventSubscribers?: (serviceId: string) => MaybePromise<ServiceEventSubscriber>;
    _eventEmitters?: (serviceId: string) => MaybePromise<ServiceEventEmitter>;
    addModule<T extends ServiceModule = any>(id: InferModuleID<T>, factory: MaybeFactory<T>): this;
    add<T extends ServiceInterface = any>(id: InferServiceID<T>, factory: MaybeFactory<Service<T>>): this;
    setEventEmitter(factory: (serviceId: string) => MaybePromise<ServiceEventEmitter>): this;
    setEventSubscriber(factory: (serviceId: string) => MaybePromise<ServiceEventSubscriber>): this;
}
export declare class ServiceInstanceBuilder {
    _serviceRegistry: ServiceRegistry;
    _serviceModuleRegistries: {
        [serviceId: string]: ServiceModuleRegistry;
    };
    _defaultModuleRegistry: ServiceModuleRegistry;
    _defaultEventSubscribers?: (serviceId: string) => MaybePromise<ServiceEventSubscriber>;
    _defaultEventEmitters?: (serviceId: string) => MaybePromise<ServiceEventEmitter>;
    constructor();
    addDefaultModule<T extends ServiceModule = any>(id: InferModuleID<T>, factory: MaybeFactory<T>): this;
    setDefaultEventEmitter(factory: (serviceId: string) => MaybePromise<ServiceEventEmitter>): this;
    setDefaultEventSubscriber(factory: (serviceId: string) => MaybePromise<ServiceEventSubscriber>): this;
    add<T extends ServiceInterface = any>(id: InferServiceID<T>, factory: MaybeFactory<Service<T>>): this;
    addGroup(group: ServiceGroup): this;
    private _resolveServices;
    createInstance(): Promise<ServiceInstance>;
}
//# sourceMappingURL=service-instance-builder.d.ts.map