import { Collection, ObjectId, WithId } from 'mongodb';
import { BaseService, Service } from '../../../core';
import { EntityComponent, EntityComponentServiceSurface } from '../interfaces/entity-component-manager';
declare type MongoDoc<TID, TValue> = {
    _id: TID | ObjectId;
} & TValue;
export declare type EntityComponentIndexKey<TValue extends Record<string, any>> = Partial<{
    [K in keyof TValue]: boolean | 1 | -1;
}>;
export interface EntityComponentServiceOptions<TValue extends Record<string, any>> {
    id: string;
    indexes?: {
        key: EntityComponentIndexKey<TValue>;
        unique?: boolean;
    }[];
}
export declare class EntityComponentService<TID extends string, TValue extends Record<string, any>> extends BaseService<EntityComponentServiceSurface<TID, TValue>> implements Service<EntityComponentServiceSurface<TID, TValue>> {
    private _module;
    private _collection;
    get collection(): Collection<MongoDoc<TID, TValue>>;
    indexedFields: Set<string>;
    options: EntityComponentServiceOptions<TValue>;
    constructor(options: EntityComponentServiceOptions<TValue>);
    createIndexes(): Promise<void>;
    protected _parseDocId(id: TID): TID | ObjectId;
    protected _toId(id: TID | ObjectId): TID;
    protected _parseEntityComponent(doc: WithId<MongoDoc<TID, TValue>>): EntityComponent<TID, TValue>;
    get(id: TID): Promise<EntityComponent<TID, TValue> | undefined>;
    getByIds(ids: TID[]): Promise<(EntityComponent<TID, TValue> | undefined)[]>;
}
export {};
//# sourceMappingURL=entity-component-service.d.ts.map